#ifndef COMMONS_H_INCLUDED
#define COMMONS_H_INCLUDED



/*
 * LIBRARIES
 */
// STD LIB
#include <iostream>
#include <cstdlib>
#include <fstream>
#include <sstream>
#include <cassert>
#include <cmath>
#include <climits>
#include <vector>
// LOCAL MODULES
#include "utils.h"



/*
 * DEFINES
 */
#define MESH_MAX_POINT MAX_INT-1
#define EPSILON 0.0001
#define FOLLOW_SEARCH



/*
 * PREDECLARATIONS
 */
// Verbose level. Greater it is, deeper and accurate logs are.
//short VERBOSE_LEVEL = 10;





#endif
